package gus05.manager.gus.kassia.layer2.tools;

import gus05.manager.gus.kassia.layer2.KassiaSystem;

import java.io.File;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JOptionPane;

public class FatalErrorHolder extends KassiaSystem {
	
	
	private File file;
	private PrintStream out;
	
	private SimpleDateFormat yyyyMMdd_HHmmss = new SimpleDateFormat("yyyyMMdd_HHmmss");
    private String timeStamp(Date date){return yyyyMMdd_HHmmss.format(date);}

    
	
	public FatalErrorHolder(String id, int exitCode, Throwable e)
	{
		String abortTime = timeStamp(new Date());
		String startTime = timeStamp(appStartDate);
			
		initFile(abortTime);
        
        if(out!=null)
        {
        	out.println("FATAL ERROR REPORT");
        	out.println("------------------");
        	out.println("manager ID: ["+MANAGERID+"]");
        	out.println("internal ID: ["+id()+"]");
        	out.println("internal state: "+state());
        	out.println("args line: ["+appArgsLine+"]");
        	out.println("launch dir: "+System.getProperty("user.dir"));
        	out.println("start time: "+startTime);
        	out.println("abort time: "+abortTime);
        	out.println("exit code: "+exitCode);
        	out.println("------------------");
        	out.println("java version: "+System.getProperty("java.version"));
        	out.println("java home: "+System.getProperty("java.home"));
        	out.println("------------------");
        	out.println("exception source "+id);
        	out.println("exception time: "+abortTime);
        	out.println("exception type: "+e.getClass().getName());
        	out.println("exception message: "+e.getMessage());
        	out.println("------------------");
        	e.printStackTrace(out);
        	out.close();
        }
        
        String m1 = "FATAL ERROR has occurred at location: "+id;
        String m2 = "Application must abort (exit code = "+exitCode+")";
        String m3 = "For further details, please read the FATAL ERROR REPORT file:";
        
        // print into err PrintStream
        System.err.println(m1);
        e.printStackTrace(System.err);
        System.err.println(m2);
        if(out!=null) System.err.println(m3);
        
        // display error dialog
        String message = m1+"\n"+e+"\n\n"+m2;
        if(out!=null) message = message+"\n"+m3+"\n"+file.getAbsolutePath();
        showErr(message,"FATAL ERROR");
        
        System.err.println("System.exit("+exitCode+")");
        System.exit(exitCode);
	}
	
	
	
	
	private void initFile(String time)
	{
		String fileName = "fatalerr_"+time+"_"+MANAGERID+".txt";
        file = new File(fileName); //file inside the launch dir
		
		try
		{
			out = new PrintStream(file);
		}
		catch(Exception e)
		{
			String message = "Failed to initialize report file:\n"+file.getAbsolutePath()+"\n"+e;
			showErr(message,"Report file failure");
		}
	}
	
	
	
	
	private void showErr(String message, String title)
	{
		JOptionPane.showMessageDialog(null,message,title,JOptionPane.ERROR_MESSAGE);
	}
}
