package gus05.manager.gus.kassia.layer3.a14.initiconbuilder;

import java.io.File;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import gus05.framework.features.Transform;
import gus05.manager.gus.kassia.layer3.KS6;
import gus05.manager.gus.kassia.layer3.tools.icon.AbstractLazyIcon;
import gus05.manager.gus.kassia.layer3.tools.icon.IconCompound;


public class A14 extends KS6 implements Transform {

	
	
	public static void initIconBuilder()
    {
		putPoolList(POOL_LIST_ICONIDS);
        putPool(POOL_TRANSFORM_ICONBUILDER,new A14());
    }
	
	
	
	public Object transform(Object obj) throws Exception
	{
		Object[] data = (Object[]) obj;
		if(data.length!=2) throw new Exception("Wrong data number: "+data.length);
		String key = (String) data[0];
		Object source = data[1];
		
		return buildIcon(key,source);
	}
    
    
    
    
    private Icon buildIcon(String key, Object source) throws Exception
    {
    	if(source==null) throw new Exception("Null source for icon name: ["+key+"]");
    	
    	if(source instanceof Icon) return (Icon) source;
    	if(source instanceof URL) return buildIcon_URL(key,(URL)source);
    	if(source instanceof File) return buildIcon_File(key,(File)source);
    	if(source instanceof String) return buildIcon_String(key,(String)source);
    	
    	throw new Exception("Invalid source type: ["+source.getClass().getName()+"] for icon name: ["+key+"]");
    }
    
    
    
    
    private Icon buildIcon_URL(String key, URL url)
    {
    	if(isIconLazyLoad())
    		return new LazyIcon_url(key,url);
    	else return loadIcon_URL(key,url);
    }
    
    
    
    private Icon buildIcon_File(String key, File file) throws Exception
    {
    	if(!file.exists())	throw new Exception("File not found for icon name: ["+key+"]");
		
		if(isIconLazyLoad())
    		return new LazyIcon_file(key,file);
    	else return loadIcon_File(key,file);
    }
    
    
    
    private Icon buildIcon_String(String key, String rule) throws Exception
    {
    	String[] n = rule.split("#",2);
    	if(n.length!=2) throw new Exception("Invalid rule: ["+rule+"] for ifon name: ["+key+"]");
    	return buildIcon_String(key,n[0],n[1]);
    }
    
    
    
    private Icon buildIcon_String(String key, String type, String info) throws Exception
    {
    	if(type.equals("redirect")) return  new Icon_lazy_redirect(key,info);
    	if(type.equals("compound")) return  new Icon_lazy_compound(key,info);
    	if(type.equals("file")) return buildIcon(key,s0_getFile(info));
    	if(type.equals("url")) return buildIcon(key,urlProp(info));
    	
    	throw new Exception("Unknown type: ["+type+"] for ifon name: ["+key+"]");
    }
    
    
    
    
    
    
    
    /*
     * On pourrait ensuite utiliser un AbstractLazyImageIcon
     * dans le cas d'une icne charge  partir d'une URL
     */
    private class LazyIcon_url extends AbstractLazyIcon
    {
    	private URL url;
    	public LazyIcon_url(String key, URL url)
    	{
    		super(key);
    		this.url = url;
    	}
    	protected Icon build()
    	{return loadIcon_URL(key(),url);}
    }
    
    
    
    
    
    private class LazyIcon_file extends AbstractLazyIcon
    {
    	private File file;
    	public LazyIcon_file(String key, File file)
    	{
    		super(key);
    		this.file = file;
    	}
    	protected Icon build()
    	{return loadIcon_File(key(),file);}
    }
    
    
    
    
    
    
    
    private class Icon_lazy_redirect extends AbstractLazyIcon
    {
    	private String rule;
    	public Icon_lazy_redirect(String key, String rule)
    	{
    		super(key);
    		this.rule = rule;
    	}
    	protected Icon build()
    	{return icon1(rule);}
    }
    
    
    
    
    
    
    private class Icon_lazy_compound extends AbstractLazyIcon
    {
    	private String[] ids;
    	public Icon_lazy_compound(String key, String rule)
    	{
    		super(key);
    		this.ids = rule.split(";");
    	}
    	protected Icon build()
    	{
    		Icon[] icons = new Icon[ids.length];
    		for(int i=0;i<ids.length;i++) icons[i] = icon1(ids[i]);
    		return new IconCompound(icons);
    	}
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private Icon loadIcon_File(String key, File file)
    {
    	Icon icon = newIcon_File(file);
    	iconIdList().add(key);
    	return icon;
    }
    
    private ImageIcon loadIcon_URL(String key, URL url)
    {
    	ImageIcon icon = newIcon_URL(url);
    	iconIdList().add(key);
    	return icon;
    }
}
