package gus05.manager.gus.kassia.layer3.g01.initresourceprovider;


import java.util.Map;

import gus05.framework.core.Entity;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.features.Transform;
import gus05.manager.gus.kassia.layer3.tools.AbstractResourceProvider;


public class G01 extends AbstractResourceProvider {

	
	
	public static void initResourceProvider() throws Exception
	{
		putPoolMap(POOL_MAP_PROVIDERS);
        putPool(POOL_RESOURCEPROVIDER,new G01());
	}
	
	
	protected Object retrieveResource(Entity entity, String id, String key) throws Exception
	{return new ResourceProvider1(entity,id).retrieve(key);}
	

	
	

	
	private class ResourceProvider1 implements Retrieve
	{
		private Entity entity;
        private String id;
        private String providerID;
        private String info;
        
        
        private String src()
        {return entity.getName()+"@"+id;}
        
        public String toString()
        {return " source=["+src()+"] providerID=["+providerID+"] info=["+info+"]";}
        
        
        
        public ResourceProvider1(Entity entity, String id)
        {
            this.entity = entity;
            this.id = id;
        }
        
        
        public Object retrieve(String key) throws Exception
        {
        	analyzeKey(key);
    		
        	try{return provideResource();}
        	catch(Exception e)
        	{return prepareAndprovideResource(e);}
        }
        
        
        
        
        
        
        
        private void analyzeKey(String key) throws Exception
        {
        	if(key==null) throw new Exception("Invalid null key");
        	if(key.equals("")) throw new Exception("Invalid empty key");	
        	
        	if(key.contains("#"))
        	{
        		String[] n = key.split("#",2);
        		providerID = n[0];
        		info = n[1];
        	}
        	else if(map_providers().containsKey(key))
        	{
        		providerID = key;
        		info = "";
        	}
        	else if(hasPool(key))
        	{
        		providerID = PROVIDER_POOL;
        		info = key;
        	}
        	else
        	{
        		providerID = getDefaultProviderID();
        		info = key;
        	}
        }
        
        
        
        
        
        
        
        
        
        private Object provideResource() throws Exception
        {
            try
            {
            	Map m = map_providers();
            	
            	if(!m.containsKey(providerID))
            		tryAndInitProvider(m,providerID);
            	
                if(!m.containsKey(providerID))
                    throw new Exception("Unknown resource provider: ["+providerID+"]");
                
            	Retrieve provider = (Retrieve) m.get(providerID);
                return provideResource(provider);
            }
            catch(Exception e)
            {
                String message = "Resource call failed: "+toString();
                throw new Exception(message,e);
            }
        }
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        private Object provideResource(Retrieve provider) throws Exception
        {
            try
            {
                if(provider instanceof Register)
                {
                    ((Register)provider).register("id",id);
                    ((Register)provider).register("entity",entity);
                }
                return provider.retrieve(info);
            }
            catch(Exception e)
            {
                String message = "Resource provider failed: ["+providerID+"]";
                throw new Exception(message,e);
            }
        }
        
        
        


        
        
        
        
        
        
        
        private Object prepareAndprovideResource(Exception e) throws Exception
        {
        	Transform t = pooltran(POOL_TRANSFORM_RSCPREPARATION);
        	if(t==null) throw e;
        	
        	printW(this,"prepareAndprovideResource(Exception)","Try and prepare resource after error: "+e);
        	Object[] data = new Object[]{entity,id,providerID,info,e};
        	String newKey = null;
        	
        	try
        	{
        		newKey = (String) t.transform(data);
            	analyzeKey(newKey);
            	return provideResource();
        	}
        	catch(Exception ee)
        	{
        		printW(this,"prepareAndprovideResource(Exception)","Resource preparation failed with new key=["+newKey+"]: err="+ee.getMessage());
            	throw e;
        	}
        }
	}
	
	
	
	
	
	
	
	
	
	

	private void tryAndInitProvider(Map m, String id) throws Exception
	{
		Object provider = buildProvider(m,id);
		if(provider!=null) m.put(id, provider);
	}
	
	
	
	private Object buildProvider(Map m, String id) throws Exception
	{
		String otherID = findOtherProviderID(id);
		if(otherID==null) return null;
		if(m.containsKey(otherID))return m.get(otherID);
		
		try
		{return s1_provideUniqueEntity(otherID);}
		catch(Exception e)
		{
			String message = "Resource provider creation failed with second ID: ["+otherID+"]";
			throw new Exception(message,e);
        }
	}
	
	
	private String findOtherProviderID(String id)
	{
		if(hasProp(PROPSTART_PROVIDERRULE+id))
			return getProp(PROPSTART_PROVIDERRULE+id);
		if(id.equals(PROVIDER_DEFAULT))
			return getDefaultProviderID();
		return null;
	}
}
