package gus05.manager.gus.kassia.layer3.tools.data;

import java.awt.event.ActionListener;
import java.io.File;
import java.io.PrintStream;
import java.util.HashSet;
import java.util.List;


import javax.swing.SwingUtilities;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.features.Support;
import gus05.framework.tools.DefaultSupport;


public abstract class AbstractTree extends DefaultTreeModel implements Support, Register, Retrieve, CellEditorListener
{
    public final static String DELIM = "\t";
    
    protected DefaultSupport sup;
    protected void send(String id) {sup.send(this,id);}
    
    protected TreePath before = null;
    protected TreePath after = null;

    protected DefaultMutableTreeNode currentNode;
    protected int currentLevel;
    
    
    protected abstract void modified(String eventID);
    protected abstract void save(Object obj) throws Exception;
    
    protected String id;
    public String toString() {return id;}
    
    
    
    protected AbstractTree(String id, String value) throws Exception
    { 
        super(null);
        this.id = id;
        sup = new DefaultSupport();
        
        currentNode = null;
        currentLevel = -1;

        String[] lines = value.split("[\n\r]+");
        for(int i=0;i<lines.length;i++)
        {
            String line = lines[i];
            if(!line.equals("")) handleLine(i,line);
        }
        if(getRoot()==null)
            setRoot(new DefaultMutableTreeNode0("<>"));
    }
    
    
    
    
    
    
    private void handleLine(int index, String line) throws Exception
    {
        String[] tab = line.split(DELIM+"+",2);
        String tagName = "";
        if(tab.length==1)tagName = tab[0];
        if(tab.length>1)tagName = tab[1];
        int level = line.length()-tagName.length();
        
        if(level>0 || index==0)
        {
            // build next node
            if(level>currentLevel+1)
                throw new Exception("next node level can not be higher than "+currentLevel+1);
            
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0(tagName);
            
            if(level<currentLevel+1)
            {
                int up = currentLevel-level+1;
                for(int j=0;j<up;j++)
                    currentNode = (DefaultMutableTreeNode0)currentNode.getParent();
            }
            
            if(currentNode==null)
                setRoot(newNode);
            else currentNode.add(newNode);
            //nodeStructureChanged(currentNode);
            
            currentNode = newNode;
            currentLevel = level;
        }
    }
    
    
    
   
    
    
    
    
    private void childUp(DefaultMutableTreeNode0 child)
    {
        before = new TreePath(child.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)child.getParent();
        if(parent==null)return;
        int index = getIndexOfChild(parent,child);
        if(index==0)
        {
            DefaultMutableTreeNode0 previous = (DefaultMutableTreeNode0)parent.getPreviousSibling();
            if(previous==null)return;
            
            removeNodeFromParent(child);
            insertNodeInto(child,previous,previous.getChildCount());
            after = new TreePath(child.getPath());
        }
        else
        {
            removeNodeFromParent(child);
            insertNodeInto(child,parent,index-1);
            after = new TreePath(child.getPath());
        }
        modified("childUp(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void childDown(DefaultMutableTreeNode0 child)
    {
        before = new TreePath(child.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)child.getParent();
        if(parent==null)return;
        int index = getIndexOfChild(parent,child);
        if(index==parent.getChildCount()-1)
        {
            DefaultMutableTreeNode0 next = (DefaultMutableTreeNode0)parent.getNextSibling();
            if(next==null)return;
            
            removeNodeFromParent(child);
            insertNodeInto(child,next,0);
            after = new TreePath(child.getPath());
        }
        else
        {
            removeNodeFromParent(child);
            insertNodeInto(child,parent,index+1);
            after = new TreePath(child.getPath());
        }
        modified("childDown(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void decreaseNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 previous = (DefaultMutableTreeNode0) node.getPreviousSibling();
        if(previous==null)return;
        
        removeNodeFromParent(node);
        insertNodeInto(node,previous,previous.getChildCount());
        after = new TreePath(node.getPath());
        
        modified("decreaseNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    private void increaseNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null)return;
        DefaultMutableTreeNode0 parent2 = (DefaultMutableTreeNode0)parent.getParent();
        if(parent2==null)return;
        
        int index = getIndexOfChild(parent2,parent)+1;
        removeNodeFromParent(node);
        insertNodeInto(node,parent2,index);
        after = new TreePath(node.getPath());
        
        modified("increaseNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    private void deleteNode(DefaultMutableTreeNode0 node)
    {
        before = new TreePath(node.getPath());
        after = null;
        
        DefaultMutableTreeNode0 nextNode = (DefaultMutableTreeNode0) node.getPreviousSibling();
        if(nextNode==null) nextNode = (DefaultMutableTreeNode0) node.getNextSibling();
        if(nextNode==null) nextNode = (DefaultMutableTreeNode0) node.getParent();
        if(nextNode==null) return;
        
        removeNodeFromParent(node);
        after = new TreePath(nextNode.getPath());
        
        modified("deleteNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    
    private void createNode(DefaultMutableTreeNode0 node)
    {
        before = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null)
        {
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
            insertNodeInto(newNode,node,0);
            after = new TreePath(newNode.getPath());
        }
        else
        {
            DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
            int index = getIndexOfChild(parent,node)+1;
            insertNodeInto(newNode,parent,index);
            after = new TreePath(newNode.getPath());
        }
        modified("createNode(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    
    private void createSibling(DefaultMutableTreeNode0 node)
    {
        before = null;
        
        DefaultMutableTreeNode0 parent = (DefaultMutableTreeNode0)node.getParent();
        if(parent==null) return;

        DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
        int index = getIndexOfChild(parent,node)+1;
        insertNodeInto(newNode,parent,index);
        after = new TreePath(newNode.getPath());

        modified("createSibling(DefaultMutableTreeNode0)");
    }    

    
    
    
    
    
    
    private void createChild(DefaultMutableTreeNode0 node)
    {
        before = null;
        DefaultMutableTreeNode0 newNode = new DefaultMutableTreeNode0("");
        insertNodeInto(newNode,node,0);
        after = new TreePath(newNode.getPath());
        modified("createChild(DefaultMutableTreeNode0)");
    }
    
    
    
    
    
    private void addChild(DefaultMutableTreeNode0 parent, DefaultMutableTreeNode0 child)
    {
        before = null;
        after = new TreePath(child.getPath());
        insertNodeInto(child,parent,0);
        modified("addChild(DefaultMutableTreeNode0,DefaultMutableTreeNode0)");
    }

    
    
    
    private void updateNode(DefaultMutableTreeNode0 node)
    {nodeChanged(node);}
    
    
    
    
    
    
    public void editingCanceled(ChangeEvent e){}



    public void editingStopped(ChangeEvent e)
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run() {modified("editingStopped(ChangeEvent)");}
        });
    }
    
    
    

    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("childUp")) 		{childUp((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("childDown")) 	{childDown((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("decreaseNode"))	{decreaseNode((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("increaseNode"))	{increaseNode((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("deleteNode"))	{deleteNode((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("createNode"))	{createNode((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("createChild"))	{createChild((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("createSibling"))	{createSibling((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("updateNode"))	{updateNode((DefaultMutableTreeNode0)obj);return;}
        if(key.equals("save")) 			{save(obj);return;}
        
        if(key.equals("addChild"))
        {
            DefaultMutableTreeNode0[] nodes = (DefaultMutableTreeNode0[])obj;
            addChild(nodes[0],nodes[1]);
            return;
        }
    }

    
    public Object retrieve(String key) throws Exception
    {
        if(key.equals("after")) return after;
        if(key.equals("before")) return before;
        if(key.equals("description")) return description();
        
        if(key.equals("keys"))
            return new String[]{"after","before","description"};
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    
    
    private void addIntoSet(HashSet set, Object node)
    {
        set.add(node.toString());
        for(int i=0;i<getChildCount(node);i++)
        addIntoSet(set,getChild(node,i));
    }
        
    
    
    
    
    public String description()
    {
        StringBuffer b = new StringBuffer();
        buildRepresentation(b,getRoot(),0);
        return b.toString();
    }
    
    
    
    private void buildRepresentation(StringBuffer b, Object node, int level)
    {
        for(int i=0;i<level;i++) b.append(DELIM);
        b.append(node.toString()+"\n");
        for(int i=0;i<getChildCount(node);i++)
            buildRepresentation(b,getChild(node,i),level+1);
    }

    
    
    
    
    


    public void addActionListener(ActionListener listener)
    {sup.addActionListener(listener);}
    
    public void removeActionListener(ActionListener listener)
    {sup.removeActionListener(listener);}
    
    public List listeners()
    {return sup.listeners();}
    
    
    
    
    
    
    
    
    
    private class DefaultMutableTreeNode0 extends DefaultMutableTreeNode
    {
        public DefaultMutableTreeNode0(Object userObject)
        {super(userObject);}
        
        public void setUserObject(Object userObject)
        {
            super.setUserObject(userObject);
            modified("setUserObject(Object)");
        }
    }

    
}
