package gus05.manager.gus.kassia.layer3.v04.initmaingui;


import gus05.manager.gus.kassia.layer3.KS6;
import gus05.manager.gus.kassia.layer3.tools.Tool_debug;
import gus05.manager.gus.kassia.layer3.tools.printstream.PrintStream_Area;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintStream;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


public class ConsolePanel extends JPanel {

	
	public static final Font FONT = new Font("Courier New", Font.PLAIN, 13);
	
	public static final Color COLOR_MAINGUI = Color.ORANGE;
	public static final Color COLOR_MAINGUIFAILED = Color.RED;
	public static final Color COLOR_LOG = Color.BLUE;
	public static final Color COLOR_DEBUG = Color.GREEN.darker();
	public static final Color COLOR_ERRORS = Color.MAGENTA.darker();
	
	public static final String TITLE_MAINGUI = "MAINGUI";
	public static final String TITLE_MAINGUIFAILED = "MAINGUI FAILED";
	public static final String TITLE_LOG = "LOG";
	public static final String TITLE_DEBUG = "DEBUG";
	public static final String TITLE_ERRORS = "ERRORS";

	
	
	private JTabbedPane tab;
	private PrintStream log;
	
	public PrintStream log() {return log;}
	
	
	
	
	
	public ConsolePanel(Exception e)
	{
		super(new BorderLayout());
		init(null,e);
	}
	
	
	public ConsolePanel(JComponent comp)
	{
		super(new BorderLayout());
		init(comp,null);
	}
	
	
	
	private void init(JComponent comp, Exception e)
	{
		tab = new JTabbedPane();
		
		if(comp!=null)
		addTab(TITLE_MAINGUI,COLOR_MAINGUI,comp);
		
		if(e!=null)
		addTab(TITLE_MAINGUIFAILED,COLOR_MAINGUIFAILED,area_mainguiFailed(e));
		
		addTab(TITLE_LOG,COLOR_LOG,area_log());
		
		addTab(TITLE_DEBUG,COLOR_DEBUG,area_debug());
		
		if(KS6.hasErrors())
		addTab(TITLE_ERRORS,COLOR_ERRORS,new ErrorsPanel());
		
		JButton button_exit = new JButton("FORCE EXIT");
		button_exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{KS6.forceExit();}
		});
		
		JPanel p_bottom = new JPanel(new GridLayout(1,1));
		p_bottom.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		p_bottom.add(button_exit);
		
		add(tab,BorderLayout.CENTER);
		add(p_bottom,BorderLayout.SOUTH);
	}
	
	
	
	
	private void addTab(String title, Color color, JComponent comp)
	{
		tab.addTab(title,KS6.scroll(comp));
		tab.setForegroundAt(tab.getComponentCount()-1,color);
	}
	
	
	
	private JTextArea area(Color color)
	{
		JTextArea area = new JTextArea();
		area.setBackground(getBackground());
		area.setMargin(new Insets(4,4,4,4));
		area.setEditable(false);
		area.setForeground(color);
		area.setFont(FONT);
		return area;
	}
	
	
	
	
	
	private JTextArea area_mainguiFailed(Exception e)
	{
		JTextArea area = area(COLOR_MAINGUIFAILED);
		PrintStream p = new PrintStream_Area(area);
		KS6.printCauses(e,p);
		p.println();
		KS6.printStackTrace(e,p);
		area.setCaretPosition(0);
		return area;
	}
	
	
	private JTextArea area_log()
	{
		JTextArea area = area(COLOR_LOG);
		log = new PrintStream_Area(area);
		return area;
	}
	
	
	
	private JTextArea area_debug()
	{
		JTextArea area = area(COLOR_DEBUG);
		PrintStream p = new PrintStream_Area(area);
		Tool_debug.printAppDebugInfo_all(p);
		area.setCaretPosition(0);
		return area;
	}
	
	
	
	
	
	
	
	
	
	
	
	private class ErrorsPanel extends JSplitPane implements ListSelectionListener
	{
		private JTextArea area;
		private JList list;
		private PrintStream p;
		
		public ErrorsPanel()
		{
			super();
			setDividerLocation(200);
			
			list = new JList(buildErrorsArray());
			list.addListSelectionListener(this);
			
			area = area(COLOR_ERRORS);
			p = new PrintStream_Area(area);
			
			setLeftComponent(new JScrollPane(list));
			setRightComponent(new JScrollPane(area));
		}
		
		public void valueChanged(ListSelectionEvent evt)
		{
			area.setText("");
			int index = list.getSelectedIndex();
			if(index<0) return;
			
			Object[] info = KS6.getError(index);
			
			Object source = info[0];
	        String id = (String) info[1];
	        Exception e = (Exception) info[2];
	        Date date = (Date) info[3];
	        
	        p.println("Source: "+KS6.name(source)+"@"+id);
	        p.println("Time: "+KS6.yyyyMMdd_HHmmss(date));
	        p.println("Type: "+e.getClass().getName());
	        p.println("Message: "+e.getMessage());
	        p.println();
	        KS6.printCauses(e,p);
			p.println();
			KS6.printStackTrace(e,p);
	        area.setCaretPosition(0);
		}
	}
	
	
	
	
	
	private String[] buildErrorsArray()
	{
		int number = KS6.errorNumber();
		String[] r = new String[number];
		for(int i=0;i<number;i++)
		{
			Object[] info = KS6.getError(i);
			Object source = info[0];
	        String id = (String) info[1];
	        r[i] = KS6.name(source)+"@"+id;
		}
		return r;
	}
}
